@@ -287,6 +287,19 @@ update_progress_label (DialogData *data)
 }
 
 
+static void
+search_completed (DialogData *data)
+{
+	char *text;
+	
+	gtk_widget_set_sensitive (data->cc_cancel_search_button, FALSE);
+	
+	text = g_strdup_printf ("%u", data->urls);
+	gtk_label_set_text (GTK_LABEL (data->progress_label), text);
+	g_free (text);
+}
+
+
 static void
 load_current_url (DialogData *data)
 {
@@ -295,8 +308,10 @@ load_current_url (DialogData *data)
 
 	update_progress_label (data);
 
-	if ((data->current == NULL) || (data->url >= data->max_images))
+	if ((data->current == NULL) || (data->url >= data->max_images)) {
+		search_completed (data);
 		return;
+	}
 
 	url = data->current->data;
 
@@ -609,15 +624,8 @@ static void
 cancel_search_button_clicked_cb (GtkWidget  *widget, 
        				 DialogData *data)
 {
-	char *text;
-	
 	cancel_search (data);
-	
-	text = g_strdup_printf ("%u", data->urls);
-	gtk_label_set_text (GTK_LABEL (data->progress_label), text);
-	g_free (text);
-	
-	gtk_widget_set_sensitive (widget, FALSE);
+	search_completed (data);	
 }
 
 
@@ -764,7 +772,9 @@ auto_fetch__search_result_saved_cb (DialogData *data,
 		dest = g_build_filename (data->tmpdir, filename, NULL);
 		g_free (filename);
 
-		copy_file_from_url (data, url, dest, auto_fetch__image_saved_cb);
+		copy_file_from_url (data, url, dest, auto_fetch__image_saved_cb)
+;
+		g_free (dest);
 	} 
 	else
 		destroy_cb (NULL, data);
@@ -806,7 +816,7 @@ fetch_cover_image_from_name (GooWindow  *window,
 	data->max_images = 1;
 	data->autofetching = TRUE;
 
-	data->tmpdir = g_strdup (get_temp_work_dir ());
+	data->tmpdir = get_temp_work_dir ();
 	ensure_dir_exists (data->tmpdir, DIR_PERM);
 
 	data->load_id = g_idle_add (auto_fetch_from_name__start_searching, data);
