@@ -21,6 +21,9 @@
  */
 
 #include <config.h>
+
+#ifdef HAVE_LINUX
+
 #include <string.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
@@ -33,8 +36,6 @@
 #include "goo-cdrom-linux.h"
 #include "glib-utils.h"
 
-#define DEFAULT_DEVICE "/dev/cdrom"
-
 static GooCdromClass *parent_class = NULL;
 
 static void goo_cdrom_linux_class_init  (GooCdromLinuxClass *class);
@@ -114,7 +115,7 @@ update_state_from_fd (GooCdrom *cdrom,
 {
 	int           state;
 	GooCdromState cdrom_state;
-
+	
 	state = ioctl (fd, CDROM_DISC_STATUS, 0);
 	if (state < 0) {
 		goo_cdrom_set_error_from_string (cdrom, _("Error reading CD"));
@@ -145,10 +146,11 @@ update_state_from_fd (GooCdrom *cdrom,
 		break;
 
 	case CDS_AUDIO:
+	case CDS_MIXED:
 		cdrom_state = GOO_CDROM_STATE_OK;
 		break;
 	default:
-		cdrom_state = GOO_CDROM_STATE_DATA_CD;
+		cdrom_state = GOO_CDROM_STATE_DATA_CD;			
 		break;
 	}
 
@@ -283,3 +285,6 @@ goo_cdrom_linux_new (const char *device)
 
 	return cdrom;
 }
+
+
+#endif /* HAVE_LINUX */
