@@ -23,25 +23,18 @@
  *
  */
 
-/* GTK - The GIMP Toolkit
+/* This file contains code taken from:
+ * 
+ * GTK - The GIMP Toolkit
  *
  * Copyright (C) 2003 Ricardo Fernandez Pascual
  * Copyright (C) 2004 Paolo Borelli
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * 
+ * and:
+ * 
+ * Volume Button / popup widget
+ * (c) copyright 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
+ * 
  */
 
 #include <config.h>
@@ -54,12 +47,13 @@
 #include "goo-stock.h"
 
 #define GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), GOO_TYPE_VOLUME_TOOL_BUTTON, GooVolumeToolButtonPrivate))
-#define FLOAT_EQUAL(a,b) (fabs (a - b) < 1e-6)
-
-#define SCALE_WIDTH 50
-#define SCALE_HEIGHT 145
-#define DEFAULT_VOLUME 50.0
+#define FLOAT_EQUAL(a,b) (fabs ((a) - (b)) < 1e-6)
 
+#define SCALE_WIDTH 25
+#define SCALE_HEIGHT 100
+#define DEFAULT_VOLUME 1.0
+#define CLICK_TIMEOUT 250
+#define SCROLL_TIMEOUT 100
 
 struct _GooVolumeToolButtonPrivate {
 	GtkWidget   *button;
@@ -77,12 +71,16 @@ struct _GooVolumeToolButtonPrivate {
 	double       step;
 	gboolean     mute;
 	double       mute_value;
+	
+	guint        timeout : 1;
+	guint32      pop_time;
+	
+	guint        down_button_scroll_id;
+	gboolean     down_button_pressed;
+	guint        up_button_scroll_id;
+	gboolean     up_button_pressed;
 };
 
-static void goo_volume_tool_button_init       (GooVolumeToolButton      *button);
-static void goo_volume_tool_button_class_init (GooVolumeToolButtonClass *klass);
-static void goo_volume_tool_button_finalize   (GObject                  *object);
-
 enum {
 	CHANGED,
         LAST_SIGNAL
@@ -91,32 +89,6 @@ enum {
 static GObjectClass *parent_class = NULL;
 static guint goo_volume_tool_button_signals[LAST_SIGNAL] = { 0 };
 
-GType
-goo_volume_tool_button_get_type (void)
-{
-	static GType type = 0;
-
-	if (type == 0) {
-		static const GTypeInfo info = {
-			sizeof (GooVolumeToolButtonClass),
-			(GBaseInitFunc) 0,
-			(GBaseFinalizeFunc) 0,
-			(GClassInitFunc) goo_volume_tool_button_class_init,
-			(GClassFinalizeFunc) 0,
-			NULL,
-			sizeof (GooVolumeToolButton),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) goo_volume_tool_button_init
-		};
-		
-		type = g_type_register_static (GTK_TYPE_TOOL_BUTTON,
-					       "GooVolumeToolButton",
-					       &info, 0);
-	}
-	
-	return type;
-}
-
 
 static gboolean
 goo_volume_tool_button_set_tooltip (GtkToolItem *tool_item,
@@ -149,7 +121,8 @@ goo_volume_tool_button_construct_contents (GooVolumeToolButton *button)
 	if (orientation == GTK_ORIENTATION_HORIZONTAL) {
 		box = gtk_hbox_new (FALSE, 0);
 		gtk_arrow_set (GTK_ARROW (priv->arrow), GTK_ARROW_DOWN, GTK_SHADOW_NONE);
-	} else {
+	} 
+	else {
 		box = gtk_vbox_new (FALSE, 0);
 		gtk_arrow_set (GTK_ARROW (priv->arrow), GTK_ARROW_RIGHT, GTK_SHADOW_NONE);
 	}
@@ -187,28 +160,83 @@ goo_volume_tool_button_construct_contents (GooVolumeToolButton *button)
 }
 
 
+static void
+update_volume_label (GooVolumeToolButton *button)
+{
+	double  value = button->priv->value;
+	char   *text;
+	char   *icon;
+
+	if ((value - 0.0) < 10e-3)
+		icon = GOO_STOCK_VOLUME_ZERO;
+	else if (value < 0.25)
+		icon = GOO_STOCK_VOLUME_MIN;
+	else if (value < 0.75)
+		icon = GOO_STOCK_VOLUME_MED;
+	else
+		icon = GOO_STOCK_VOLUME_MAX;
+
+	gtk_tool_button_set_stock_id (GTK_TOOL_BUTTON (button), icon);
+	
+	text = g_strdup_printf ("<small>%3.0f%%</small>", value * 100.0);
+	gtk_label_set_markup (GTK_LABEL (button->priv->volume_label), text);
+	g_free (text);
+
+	text = g_strdup_printf (_("Volume level: %3.0f%%"), value * 100.0);
+	gtk_tooltips_set_tip (button->priv->tips,
+			      button->priv->button,
+			      text,
+			      NULL);
+	g_free (text);
+}
+
+
 static void
 goo_volume_tool_button_toolbar_reconfigured (GtkToolItem *toolitem)
 {
 	goo_volume_tool_button_construct_contents (GOO_VOLUME_TOOL_BUTTON (toolitem));
-
+	
 	/* chain up */
 	GTK_TOOL_ITEM_CLASS (parent_class)->toolbar_reconfigured (toolitem);
 }
 
 
+static void
+goo_volume_tool_button_init (GooVolumeToolButton *button)
+{
+	button->priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
+	button->priv->mute = FALSE;
+	button->priv->mute_value = 0.0;
+	button->priv->value = 0.0;
+	button->priv->step = 0.05;
+}
+
+
+static void
+goo_volume_tool_button_finalize (GObject *object)
+{
+	GooVolumeToolButton *button;
+
+	button = GOO_VOLUME_TOOL_BUTTON (object);
+	gtk_object_unref (GTK_OBJECT (button->priv->tips));
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
 static void
 goo_volume_tool_button_class_init (GooVolumeToolButtonClass *klass)
 {
-	GObjectClass *object_class;
-	GtkToolItemClass *toolitem_class;
+	GObjectClass       *object_class;
+	GtkToolItemClass   *toolitem_class;
 	GtkToolButtonClass *toolbutton_class;
+	GtkWidgetClass     *gtkwidget_class;
 	
 	parent_class = g_type_class_peek_parent (klass);
   
-	object_class = (GObjectClass *)klass;
-	toolitem_class = (GtkToolItemClass *)klass;
-	toolbutton_class = (GtkToolButtonClass *)klass;
+	object_class = (GObjectClass *) klass;
+	toolitem_class = (GtkToolItemClass *) klass;
+	toolbutton_class = (GtkToolButtonClass *) klass;
+	gtkwidget_class = (GtkWidgetClass *) klass;
 
 	goo_volume_tool_button_signals[CHANGED] =
                 g_signal_new ("changed",
@@ -223,11 +251,38 @@ goo_volume_tool_button_class_init (GooVolumeToolButtonClass *klass)
 	object_class->finalize = goo_volume_tool_button_finalize;
 	toolitem_class->set_tooltip = goo_volume_tool_button_set_tooltip;
 	toolitem_class->toolbar_reconfigured = goo_volume_tool_button_toolbar_reconfigured;
-
+	
 	g_type_class_add_private (object_class, sizeof (GooVolumeToolButtonPrivate));
 }
 
 
+GType
+goo_volume_tool_button_get_type (void)
+{
+	static GType type = 0;
+
+	if (type == 0) {
+		static const GTypeInfo info = {
+			sizeof (GooVolumeToolButtonClass),
+			(GBaseInitFunc) 0,
+			(GBaseFinalizeFunc) 0,
+			(GClassInitFunc) goo_volume_tool_button_class_init,
+			(GClassFinalizeFunc) 0,
+			NULL,
+			sizeof (GooVolumeToolButton),
+			0, /* n_preallocs */
+			(GInstanceInitFunc) goo_volume_tool_button_init
+		};
+		
+		type = g_type_register_static (GTK_TYPE_TOOL_BUTTON,
+					       "GooVolumeToolButton",
+					       &info, 0);
+	}
+	
+	return type;
+}
+
+
 static void
 button_state_changed_cb (GtkWidget           *widget,
                          GtkStateType         previous_state,
@@ -247,9 +302,11 @@ button_state_changed_cb (GtkWidget           *widget,
 	
 	if (state == GTK_STATE_PRELIGHT) {
 		gtk_widget_set_state (other, state);
-	} else if (state == GTK_STATE_NORMAL) {
+	} 
+	else if (state == GTK_STATE_NORMAL) {
 		gtk_widget_set_state (other, state);
-	} else if (state == GTK_STATE_ACTIVE) {
+	} 
+	else if (state == GTK_STATE_ACTIVE) {
 		gtk_widget_set_state (other, GTK_STATE_NORMAL);
 	}
 
@@ -259,38 +316,6 @@ button_state_changed_cb (GtkWidget           *widget,
 }
 
 
-static void
-update_volume_label (GooVolumeToolButton *button)
-{
-	double     value = button->priv->value;
-	char      *text;
-	char      *stock_id;
-
-	if ((value - 0.0) < 10e-3)
-		stock_id = GOO_STOCK_VOLUME_ZERO;
-	else if (value < 25.0)
-		stock_id = GOO_STOCK_VOLUME_MIN;
-	else if (value < 75.0)
-		stock_id = GOO_STOCK_VOLUME_MED;
-	else
-		stock_id = GOO_STOCK_VOLUME_MAX;
-
-	gtk_tool_button_set_stock_id  (GTK_TOOL_BUTTON (button), stock_id);
-
-	text = g_strdup_printf ("%3.0f%%", value);
-	gtk_label_set_text (GTK_LABEL (button->priv->volume_label), text);
-	g_free (text);
-
-	text = g_strdup_printf (_("Volume level: %3.0f%%"), 
-				button->priv->value);
-	gtk_tooltips_set_tip (button->priv->tips,
-			      GTK_WIDGET (button->priv->button),
-			      text,
-			      NULL);
-	g_free (text);
-}
-
-
 static gboolean
 button_clicked_cb (GtkWidget           *widget,
 		   GooVolumeToolButton *button)
@@ -300,7 +325,8 @@ button_clicked_cb (GtkWidget           *widget,
 		goo_volume_tool_button_set_volume (button,
 						   0.0,
 						   TRUE);
-	} else {
+	} 
+	else {
 		if (FLOAT_EQUAL (button->priv->mute_value, 0.0))
 			button->priv->mute_value = DEFAULT_VOLUME;
 		goo_volume_tool_button_set_volume (button,
@@ -312,42 +338,80 @@ button_clicked_cb (GtkWidget           *widget,
 }
 
 
-static void
-arrow_button_toggled_cb (GtkToggleButton     *toggle_button,
-			 GooVolumeToolButton *button)
+static gboolean
+arrow_button_press_cb (GtkToggleButton     *toggle_button,
+		       GdkEventButton      *event,
+		       GooVolumeToolButton *button)
 {
 	GooVolumeToolButtonPrivate *priv;
+	GtkWidget      *widget;
+	GtkAdjustment  *adj;
+  	int             x, y, m, dx, dy, sx, sy, ystartoff, mouse_y;
+  	float           v;
+  	GdkEventButton *e;
 
+	gtk_toggle_button_set_active (toggle_button, TRUE);
+  	
 	priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
+	
+	widget = GTK_WIDGET (toggle_button);
+	adj = gtk_range_get_adjustment (GTK_RANGE (priv->volume_scale));
+	
+	priv->timeout = TRUE;
 
-	if (gtk_toggle_button_get_active (toggle_button)) {
-		GtkWidget     *widget = GTK_WIDGET (toggle_button);
-		GtkAllocation  allocation = widget->allocation;
-		int            root_x, root_y;
+    	gtk_window_set_screen (GTK_WINDOW (priv->popup_win), gtk_widget_get_screen (widget));
+  
+  	gdk_window_get_origin (widget->window, &x, &y);
+  	x += widget->allocation.x;
+  	y += widget->allocation.y;
+  
+	gtk_window_move (GTK_WINDOW (priv->popup_win),
+			 x, y - (SCALE_HEIGHT / 2));
+	update_volume_label (button);
+	gtk_widget_show_all (priv->popup_win);
 
-		gdk_window_get_origin (widget->window, &root_x, &root_y);
-		gtk_window_move (GTK_WINDOW (priv->popup_win),
-				 root_x + allocation.x,
-				 root_y + allocation.y + allocation.height);
-		update_volume_label (button);
+	gdk_window_get_origin (priv->popup_win->window, &dx, &dy);
+  	dy += priv->popup_win->allocation.y;
+  	gdk_window_get_origin (priv->volume_scale->window, &sx, &sy);
+  	sy += priv->volume_scale->allocation.y;
+  	ystartoff = sy - dy;
+  	mouse_y = event->y;
 
-		gtk_widget_show_all (priv->popup_win);
-		gtk_window_resize (GTK_WINDOW (priv->popup_win),
-				   SCALE_WIDTH,
-				   SCALE_HEIGHT);
+	v = gtk_range_get_value (GTK_RANGE (priv->volume_scale)) / (adj->upper - adj->lower);
+  	x += (widget->allocation.width - priv->popup_win->allocation.width) / 2;
+  	y -= ystartoff;
+  	y -= GTK_RANGE (priv->volume_scale)->min_slider_size / 2;
+  	m = priv->volume_scale->allocation.height - GTK_RANGE (priv->volume_scale)->min_slider_size;
+  	y -= m * (1.0 - v);
+  	y += mouse_y;
+  	gtk_window_move (GTK_WINDOW (priv->popup_win), x, y);
+  	gdk_window_get_origin (priv->volume_scale->window, &sx, &sy);
+	
+	gdk_pointer_grab (priv->popup_win->window, 
+			  TRUE,
+			  (GDK_POINTER_MOTION_MASK 
+			   | GDK_BUTTON_PRESS_MASK 
+			   | GDK_BUTTON_RELEASE_MASK),
+			  NULL, 
+			  NULL, 
+			  GDK_CURRENT_TIME);
+	gdk_keyboard_grab (priv->popup_win->window, TRUE, GDK_CURRENT_TIME);
+	gtk_widget_grab_focus (priv->volume_scale);
+	gtk_grab_add (priv->popup_win);
 
-		gdk_pointer_grab (priv->popup_win->window, 
-				  TRUE,
-				  (GDK_POINTER_MOTION_MASK 
-				   | GDK_BUTTON_PRESS_MASK 
-				   | GDK_BUTTON_RELEASE_MASK),
-				  NULL, 
-				  NULL, 
-				  GDK_CURRENT_TIME);
-		gdk_keyboard_grab (priv->popup_win->window, TRUE, GDK_CURRENT_TIME);
-		gtk_widget_grab_focus (priv->volume_scale);
-		gtk_grab_add (priv->popup_win);
-	} 
+	/* forward event to the slider */
+	e = (GdkEventButton *) gdk_event_copy ((GdkEvent *) event);
+	e->window = priv->volume_scale->window;
+	e->x = priv->volume_scale->allocation.width / 2;
+	m = priv->volume_scale->allocation.height - GTK_RANGE (priv->volume_scale)->min_slider_size;
+	e->y = ((1.0 - v) * m) + GTK_RANGE (priv->volume_scale)->min_slider_size / 2 + 1;
+	gtk_widget_event (priv->volume_scale, (GdkEvent *) e);
+	e->window = event->window;
+	gdk_event_free ((GdkEvent *) e);		
+
+	priv->pop_time = event->time;
+
+	return TRUE;
 }
 
 
@@ -358,6 +422,7 @@ ungrab (GooVolumeToolButton *button)
 	gdk_keyboard_ungrab (GDK_CURRENT_TIME);
 	gtk_grab_remove (button->priv->popup_win);
 	gtk_widget_hide (button->priv->popup_win);
+	
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button->priv->arrow_button), FALSE);
 }
 
@@ -377,6 +442,27 @@ volume_scale_value_changed_cb (GtkRange            *range,
 }
 
 
+static gboolean
+scale_button_release_cb (GtkToggleButton     *toggle_button,
+		         GdkEventButton      *event,
+		         GooVolumeToolButton *button)
+{
+	GooVolumeToolButtonPrivate *priv;
+	priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
+
+	if (priv->timeout) {
+		/* if we did a quick click, leave the window open; else, hide it */
+		if (event->time > priv->pop_time + CLICK_TIMEOUT) {
+			ungrab (button);
+			return FALSE;
+		}
+		priv->timeout = FALSE;
+	}
+	
+	return FALSE;
+}
+
+
 static int
 popup_win_event_cb (GtkWidget           *widget, 
 		    GdkEvent            *event, 
@@ -395,11 +481,13 @@ popup_win_event_cb (GtkWidget           *widget,
 		    || (event_widget == button->priv->arrow_button)) {
 			ungrab (button);
 			return TRUE;
-		} else {
+		} 
+		else {
 			int x, y, w, h;
+			
 			gdk_window_get_geometry (priv->popup_win->window, 
 						 &x, &y, &w, &h, NULL);
-			
+
 			if ((event->button.x < 0) 
 			    || (event->button.x > w)
 			    || (event->button.y < 0) 
@@ -448,15 +536,179 @@ button_scroll_event_cb (GtkWidget           *widget,
 					   TRUE);
 
 	return TRUE;
+}
 
+
+/* down button */
+
+
+static gboolean
+down_button_scroll (gpointer data)
+{
+	GooVolumeToolButton *button = data;
+
+	goo_volume_tool_button_set_volume (button, 
+					   button->priv->value - button->priv->step,
+					   TRUE);
+	return TRUE;
+}
+
+
+static void
+down_button_pressed_cb (GtkButton *gtk_button,
+                        gpointer   user_data)
+{
+	GooVolumeToolButton *button = user_data;
+	
+	button->priv->down_button_pressed = TRUE;
+	if (button->priv->down_button_scroll_id != 0)
+		return;
+	button->priv->down_button_scroll_id = g_timeout_add (SCROLL_TIMEOUT, down_button_scroll, button); 
+}
+
+
+static void
+down_button_released_cb (GtkButton *gtk_button,
+                         gpointer   user_data)
+{
+	GooVolumeToolButton *button = user_data;
+	
+	button->priv->down_button_pressed = FALSE;
+	if (button->priv->down_button_scroll_id == 0)
+		return;
+	g_source_remove (button->priv->down_button_scroll_id);
+	button->priv->down_button_scroll_id = 0;
+}
+
+
+static void
+down_button_enter_cb (GtkButton *gtk_button,
+                      gpointer   user_data)
+{
+	GooVolumeToolButton *button = user_data;
+	
+	if (! button->priv->down_button_pressed)
+		return;
+	if (button->priv->down_button_scroll_id != 0)
+		return;
+	button->priv->down_button_scroll_id = g_timeout_add (SCROLL_TIMEOUT, down_button_scroll, button); 
+}
+
+
+static void
+down_button_leave_cb (GtkButton *gtk_button,
+                      gpointer   user_data)
+{
+	GooVolumeToolButton *button = user_data;
+	
+	if (button->priv->down_button_scroll_id == 0) 
+		return;
+	g_source_remove (button->priv->down_button_scroll_id);
+	button->priv->down_button_scroll_id = 0;
+}
+
+
+static void
+down_button_clicked_cb (GtkButton *gtk_button,
+                        gpointer   user_data)
+{
+	GooVolumeToolButton *button = user_data;
+	
+	button->priv->down_button_pressed = FALSE;
+	down_button_scroll (button);
+	if (button->priv->down_button_scroll_id != 0) { 
+		g_source_remove (button->priv->down_button_scroll_id);
+		button->priv->down_button_scroll_id = 0;
+	}
+}
+
+
+/* up button */
+
+
+static gboolean
+up_button_scroll (gpointer data)
+{
+	GooVolumeToolButton *button = data;
+	
+	goo_volume_tool_button_set_volume (button, 
+					   button->priv->value + button->priv->step,
+					   TRUE);
+	return TRUE;
+}
+
+
+static void
+up_button_pressed_cb (GtkButton *gtk_button,
+                      gpointer   user_data)
+{
+	GooVolumeToolButton *button = user_data;
+	
+	button->priv->up_button_pressed = TRUE;
+	if (button->priv->up_button_scroll_id != 0)
+		return;
+	button->priv->up_button_scroll_id = g_timeout_add (SCROLL_TIMEOUT, up_button_scroll, button); 
+}
+
+
+static void
+up_button_released_cb (GtkButton *gtk_button,
+                       gpointer   user_data)
+{
+	GooVolumeToolButton *button = user_data;
+	
+	button->priv->up_button_pressed = FALSE;
+	if (button->priv->up_button_scroll_id == 0)
+		return;
+	g_source_remove (button->priv->up_button_scroll_id);
+	button->priv->up_button_scroll_id = 0;
+}
+
+
+static void
+up_button_enter_cb (GtkButton *gtk_button,
+                    gpointer   user_data)
+{
+	GooVolumeToolButton *button = user_data;
+	
+	if (! button->priv->up_button_pressed)
+		return;
+	if (button->priv->up_button_scroll_id != 0)
+		return;
+	button->priv->up_button_scroll_id = g_timeout_add (SCROLL_TIMEOUT, up_button_scroll, button); 
+}
+
+
+static void
+up_button_leave_cb (GtkButton *gtk_button,
+                    gpointer   user_data)
+{
+	GooVolumeToolButton *button = user_data;
+	
+	if (button->priv->up_button_scroll_id == 0) 
+		return;
+	g_source_remove (button->priv->up_button_scroll_id);
+	button->priv->up_button_scroll_id = 0;
+}
+
+
+static void
+up_button_clicked_cb (GtkButton *gtk_button,
+                      gpointer   user_data)
+{
+	GooVolumeToolButton *button = user_data;
+	
+	button->priv->up_button_pressed = FALSE;
+	up_button_scroll (button);
+	if (button->priv->up_button_scroll_id != 0) { 
+		g_source_remove (button->priv->up_button_scroll_id);
+		button->priv->up_button_scroll_id = 0;
+	}
 }
 
 
 static void 
-goo_volume_button_construct (GooVolumeToolButton *button,
-			     double               from_value,
-			     double               to_value,
-			     double               step)
+goo_volume_button_construct (GooVolumeToolButton *button)
 {
 	struct _GooVolumeToolButtonPrivate *priv = button->priv;
 	GtkWidget *box;
@@ -465,8 +717,9 @@ goo_volume_button_construct (GooVolumeToolButton *button,
 	GtkWidget *real_button;
 	GtkWidget *out_frame;
 	GtkWidget *volume_vbox;
-	GtkWidget *label;
-
+	GtkWidget *up_button;
+	GtkWidget *down_button;
+	
 	priv->tips = gtk_tooltips_new ();
 	gtk_object_ref (GTK_OBJECT (priv->tips));
 	gtk_object_sink (GTK_OBJECT (priv->tips));
@@ -491,24 +744,71 @@ goo_volume_button_construct (GooVolumeToolButton *button,
 	gtk_container_set_border_width (GTK_CONTAINER (volume_vbox), 0);
 	gtk_container_add (GTK_CONTAINER (out_frame), volume_vbox);
 
-	label = gtk_label_new (_("+"));
-	gtk_box_pack_start (GTK_BOX (volume_vbox), label, FALSE, FALSE, 0);
+	up_button = gtk_button_new_with_label (_("+"));
+	gtk_button_set_relief (GTK_BUTTON (up_button), GTK_RELIEF_NONE);
+	g_signal_connect (up_button, 
+			  "pressed",
+			  G_CALLBACK (up_button_pressed_cb), 
+			  button);
+	g_signal_connect (up_button, 
+			  "released",
+			  G_CALLBACK (up_button_released_cb), 
+			  button);
+	g_signal_connect (up_button, 
+			  "enter",
+			  G_CALLBACK (up_button_enter_cb), 
+			  button);
+	g_signal_connect (up_button, 
+			  "leave",
+			  G_CALLBACK (up_button_leave_cb), 
+			  button);
+	g_signal_connect (up_button, 
+			  "clicked",
+			  G_CALLBACK (up_button_clicked_cb), 
+			  button);
+	gtk_box_pack_start (GTK_BOX (volume_vbox), up_button, FALSE, FALSE, 0);
 
-	priv->volume_scale = gtk_vscale_new_with_range (from_value, to_value, step);
+	priv->volume_scale = gtk_vscale_new_with_range (0.0, 1.0, 0.1);
 	gtk_range_set_inverted (GTK_RANGE (priv->volume_scale), TRUE);
 	gtk_scale_set_draw_value (GTK_SCALE (priv->volume_scale), FALSE);
 	/*gtk_range_set_update_policy (GTK_RANGE (priv->volume_scale), GTK_UPDATE_DELAYED);*/
-	gtk_range_set_increments (GTK_RANGE (priv->volume_scale), step, step);
-
+	gtk_range_set_increments (GTK_RANGE (priv->volume_scale), 0.1, 0.1);
+	gtk_widget_set_size_request (priv->volume_scale, -1, SCALE_HEIGHT);
+	 
 	gtk_box_pack_start (GTK_BOX (volume_vbox), priv->volume_scale, TRUE, TRUE, 0);
 
 	g_signal_connect (priv->volume_scale, 
 			  "value_changed",
 			  G_CALLBACK (volume_scale_value_changed_cb), 
 			  button);
-
-	label = gtk_label_new (_("-"));
-	gtk_box_pack_start (GTK_BOX (volume_vbox), label, FALSE, FALSE, 0);
+	g_signal_connect (priv->volume_scale, 
+			  "button_release_event",
+			  G_CALLBACK (scale_button_release_cb), 
+			  button);
+			  
+	down_button = gtk_button_new_with_label (_("-"));
+	gtk_button_set_relief (GTK_BUTTON (down_button), GTK_RELIEF_NONE);
+	g_signal_connect (down_button, 
+			  "pressed",
+			  G_CALLBACK (down_button_pressed_cb), 
+			  button);
+	g_signal_connect (down_button, 
+			  "released",
+			  G_CALLBACK (down_button_released_cb), 
+			  button);
+	g_signal_connect (down_button, 
+			  "enter",
+			  G_CALLBACK (down_button_enter_cb), 
+			  button);
+	g_signal_connect (down_button, 
+			  "leave",
+			  G_CALLBACK (down_button_leave_cb), 
+			  button);
+	g_signal_connect (down_button, 
+			  "clicked",
+			  G_CALLBACK (down_button_clicked_cb), 
+			  button);
+	gtk_box_pack_start (GTK_BOX (volume_vbox), down_button, FALSE, FALSE, 0);
 
 	/**/
 
@@ -538,7 +838,8 @@ goo_volume_button_construct (GooVolumeToolButton *button,
 	gtk_tooltips_set_tip (priv->tips, arrow_button, _("Change the volume level"), NULL);
 
 	gtk_widget_show_all (box);
-
+	gtk_widget_hide (button->priv->volume_label);
+	
 	gtk_container_add (GTK_CONTAINER (button), box);
 
 	button->priv->button = real_button;
@@ -560,8 +861,8 @@ goo_volume_button_construct (GooVolumeToolButton *button,
 			  G_CALLBACK (button_clicked_cb), 
 			  button);
 	g_signal_connect (arrow_button, 
-			  "toggled",
-			  G_CALLBACK (arrow_button_toggled_cb), 
+			  "button_press_event",
+			  G_CALLBACK (arrow_button_press_cb), 
 			  button);
 
 	g_signal_connect (G_OBJECT (button->priv->button),
@@ -571,39 +872,15 @@ goo_volume_button_construct (GooVolumeToolButton *button,
 }
 
 
-static void
-goo_volume_tool_button_init (GooVolumeToolButton *button)
-{
-	button->priv = GOO_VOLUME_TOOL_BUTTON_GET_PRIVATE (button);
-	button->priv->mute = FALSE;
-	button->priv->mute_value = 0.0;
-}
-
-
-static void
-goo_volume_tool_button_finalize (GObject *object)
-{
-	GooVolumeToolButton *button;
-
-	button = GOO_VOLUME_TOOL_BUTTON (object);
-
-	gtk_object_unref (GTK_OBJECT (button->priv->tips));
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
-
 GtkToolItem *
-goo_volume_tool_button_new (double from_value,
-			    double to_value,
-			    double step)
+goo_volume_tool_button_new (void)
 {
 	GooVolumeToolButton *button;
 
 	button = g_object_new (GOO_TYPE_VOLUME_TOOL_BUTTON, NULL);
 
 	gtk_tool_button_set_label (GTK_TOOL_BUTTON (button), _("Volume"));
-	goo_volume_button_construct (GOO_VOLUME_TOOL_BUTTON (button), from_value, to_value, step);
+	goo_volume_button_construct (GOO_VOLUME_TOOL_BUTTON (button));
 
 	return GTK_TOOL_ITEM (button);
 }
@@ -621,8 +898,7 @@ goo_volume_tool_button_set_volume (GooVolumeToolButton *button,
 				   double               vol,
 				   gboolean             notify)
 {
-	vol = CLAMP (vol, 0.0, 100.0);
-	button->priv->value = vol;
+	button->priv->value = CLAMP (vol, 0.0, 1.0);
 	button->priv->mute = FLOAT_EQUAL (button->priv->value, 0.0);
 
 	g_signal_handlers_block_by_data (button->priv->volume_scale, button);
