@@ -65,6 +65,7 @@
 #define CONFIG_KEY_AUTOFETCH_GROUP "AutoFetch"
 #define VOLUME_BUTTON_POSITION 4
 #define TRAY_TOOLTIP_DELAY 500
+#define AUTOPLAY_DELAY 250
 
 struct _GooWindowPrivateData {
 	GtkUIManager      *ui;
@@ -119,6 +120,7 @@ struct _GooWindowPrivateData {
 	int                pos_x, pos_y;
 	gboolean           hibernate;
 	gboolean           notify_action;
+	gboolean           ejected;
 };
 
 enum {
@@ -268,6 +270,7 @@ window_update_sensitivity (GooWindow *window)
 	set_sensitive (window, "Prev", audio_cd);
 
 	set_sensitive (window, "Extract", audio_cd && (priv->album->n_tracks > 0));
+	set_sensitive (window, "CopyDisc", audio_cd && (priv->album->n_tracks > 0));
 	set_sensitive (window, "Properties", audio_cd);
 	set_sensitive (window, "PickCoverFromDisk", audio_cd);
 	set_sensitive (window, "RemoveCover", audio_cd);
@@ -279,6 +282,7 @@ window_update_sensitivity (GooWindow *window)
 	gtk_widget_set_sensitive (priv->list_view, audio_cd);
 
 	set_sensitive (window, "Eject", ! priv->hibernate);
+	set_sensitive (window, "EjectToolBar", ! priv->hibernate);
 	set_sensitive (window, "Preferences", ! priv->hibernate);
 }
 
@@ -507,6 +511,7 @@ goo_window_update_titles (GooWindow *window)
 			track_info_ref (new_track);
 			priv->current_track = new_track;
 		}
+		track_info_unref (new_track);
 
 	} while (gtk_tree_model_iter_next (model, &iter));
 }
@@ -540,10 +545,12 @@ goo_window_finalize (GObject *object)
 			g_source_remove (priv->tray_tooltip_delay);
 			priv->tray_tooltip_delay = 0;
 		}
+		
 		if (priv->next_timeout_handle != 0) {
 			g_source_remove (priv->next_timeout_handle);
 			priv->next_timeout_handle = 0;
 		}
+		
 		if (priv->update_timeout_handle != 0) {
 			g_source_remove (priv->update_timeout_handle);
 			priv->update_timeout_handle = 0;
@@ -1058,7 +1065,7 @@ goo_window_show (GtkWidget *widget)
 	set_active (window, "ViewToolbar", view_foobar);
 	goo_window_set_toolbar_visibility (window, view_foobar);
 
-	view_foobar = eel_gconf_get_boolean (PREF_UI_STATUSBAR, TRUE);
+	view_foobar = eel_gconf_get_boolean (PREF_UI_STATUSBAR, FALSE);
 	set_active (window, "ViewStatusbar", view_foobar);
 	goo_window_set_statusbar_visibility (window, view_foobar);
 
@@ -1442,7 +1449,7 @@ window_update_title (GooWindow *window)
 	else if (state == GOO_PLAYER_STATE_ERROR) {
 		GError *error = goo_player_get_error (priv->player);
 		
-		g_string_append (title, g_strdup (error->message));
+		g_string_append (title, error->message);
 		g_error_free (error);
 	} 
 	else if (state == GOO_PLAYER_STATE_NO_DISC) {
@@ -1563,6 +1570,7 @@ save_config_file (GKeyFile *kv_file,
 	else
 		g_error_free (error);
 
+	g_free (buffer);
 	gnome_vfs_close (handle);
 }
 
@@ -1656,7 +1664,7 @@ auto_fetch_cover_image (GooWindow *window)
 		g_free (filename);
 		return;
 	}
-
+	g_free (filename);
 	if (window->priv->album->asin != NULL) 
 		fetch_cover_image_from_asin (window, window->priv->album->asin);
 	else if ((window->priv->album->title != NULL) && (window->priv->album->artist != NULL))
@@ -1704,10 +1712,12 @@ player_done_cb (GooPlayer       *player,
 		goo_window_update_cover (window);
 		window_update_title (window);
 		set_current_track_icon (window, NULL);
-		if (AutoPlay) {
+		if (AutoPlay || (priv->ejected && eel_gconf_get_boolean (PREF_GENERAL_AUTOPLAY, TRUE))) {
 			AutoPlay = FALSE;
-			g_idle_add (autoplay_cb, window);
+			g_timeout_add (AUTOPLAY_DELAY, autoplay_cb, window);
 		}
+		if (goo_player_get_state (player) >= GOO_PLAYER_STATE_NO_DISC)
+			priv->ejected = FALSE;
 		break;
 
 	case GOO_PLAYER_ACTION_METADATA:
@@ -1740,8 +1750,10 @@ player_done_cb (GooPlayer       *player,
 		} 
 		else if (action == GOO_PLAYER_ACTION_STOP) 
 			set_current_track_icon (window, GTK_STOCK_MEDIA_STOP);
-		else if (action == GOO_PLAYER_ACTION_EJECT) 
+		else if (action == GOO_PLAYER_ACTION_EJECT) {
+			priv->ejected = TRUE;
 			goo_player_update (priv->player);
+		}
 		break;
 		
 	case GOO_PLAYER_ACTION_PAUSE:
